#include <stdio.h>

/*******************************************************************************/
/* Pomocne funkcije                                                            */
/*                                                                             */
/* Ovdje se koriste pointeri na funkcije sto NIJE gradivo kolokvija.           */
/* Ono sto JE gradivo kolokvija, navedeno je u poglavljima Grupa i, i=1,2,3,4. */
/*******************************************************************************/

#include <stdlib.h>

/* stdlib koristimo zbog konstante NULL, koja nam treba u pomocnoj funkciji report */

void print_niz(int niz[], int n) {
  int i;
  printf("%2d", niz[0]);
  for (i = 1; i < n; ++i) printf(",%3d", niz[i]);
  printf("\n");
}

void print_kriterij(int niz[], int n, int(*f)(int)) {
  int i;
  printf("%2d", f(niz[0]));
  for (i = 1; i < n; ++i) printf(",%3d", f(niz[i]));
  printf("\n");
}

void sort(int niz[], int n, int(*f)(int)) {
  int i, j;
  for (i = 0; i < n; ++i)
    for (j = i+1; j < n; ++j)
      if ((f && f(niz[i]) > f(niz[j])) || (!f && niz[i] > niz[j])) {
        int t = niz[i];
        niz[i] = niz[j];
        niz[j] = t;
      }
}

void report(char grupa[], int niz[], int n, int x1, int x2, int(*f)(int), int(*trazia)(int*, int, int), int(*trazib)(int*, int, int)) {
  sort(niz, n, NULL); /* Ovaj sort je ukras */
  printf("%s grupa:\n", grupa);
  printf("  Niz:                            "); print_niz(niz, n);
  printf("  Vrijednosti po kojima se trazi: "); print_kriterij(niz, n, f);
  printf("  Sekvencijalno trazenje:\n");
  printf("    Index elementa za x == %3d: %3d\n", x1, trazia(niz, n, x1));
  printf("    Index elementa za x == %3d: %3d\n", x2, trazia(niz, n, x2));
  sort(niz, n, f); /* Sort je nuzan preduvjet za binarno trazenje */
  printf("  Sortirani niz:                  "); print_niz(niz, n);
  printf("  Vrijednosti po kojima se trazi: "); print_kriterij(niz, n, f);
  printf("  Binarno trazenje:\n");
  printf("    Index elementa za x == %3d: %3d\n", x1, trazib(niz, n, x1));
  printf("    Index elementa za x == %3d: %3d\n", x2, trazib(niz, n, x2));
}

/***********/
/* Grupa 1 */
/***********/

int f1(int x) {
  /* funkcija vraca sumu razlicitih prostih djelitelja broja x */
  int sum = 0, p = 2;
  while (x > 1) {
    if (x % p == 0) {
      sum += p;
      while (x % p == 0) x /= p;
    }
    ++p;
  }
  return sum;
}

int trazi1a(int a[], int n, int x) {
  /* sekvencijalno trazenje */
  int i;
  for (i = 0; i < n; ++i)
    if (f1(a[i]) == x) return i;
  return -1;
}

int trazi1b(int a[], int n, int x) {
  /* binarno trazenje */
  /* Uvjet na niz: elementi niza moraju biti sortirani uzlazno, prema sumi razlicitih prostih djelitelja */
  int lijevi = 0, desni = n - 1, srednji, fa;
  while (lijevi <= desni) {
    srednji = (lijevi + desni) / 2;
    fa = f1(a[srednji]);
    if (x == fa) return srednji;
    if (x < fa) desni = srednji - 1; else lijevi = srednji + 1;
  }
  return -1;
}

/***********/
/* Grupa 2 */
/***********/

int f2(int x) {
  /* funkcija vraca produkt kratnosti svih prostih djelitelja broja x */
  int prod = 1, p = 2;
  while (x > 1) {
    if (x % p == 0) {
      int kratnost = 0;
      while (x % p == 0) {
        x /= p;
        ++kratnost;
      }
      prod *= kratnost;
    }
    ++p;
  }
  return prod;
}

int trazi2a(int a[], int n, int x) {
  /* sekvencijalno trazenje */
  int i;
  for (i = 0; i < n; ++i)
    if (f2(a[i]) == x) return i;
  return -1;
}

int trazi2b(int a[], int n, int x) {
  /* binarno trazenje */
  /* Uvjet na niz: elementi niza moraju biti sortirani uzlazno, prema produktu kratnosti prostih djelitelja */
  int lijevi = 0, desni = n - 1, srednji, fa;
  while (lijevi <= desni) {
    srednji = (lijevi + desni) / 2;
    fa = f2(a[srednji]);
    if (x == fa) return srednji;
    if (x < fa) desni = srednji - 1; else lijevi = srednji + 1;
  }
  return -1;
}

/***********/
/* Grupa 3 */
/***********/

int f3(int x) {
  /* funkcija vraca broj razlicitih prostih djelitelja broja x umanjen za najmanji prosti djelitelj broja x */
  int m = 0, p = 2, min = 0;
  while (x > 1) {
    if (x % p == 0) {
      while (x % p == 0) x /= p;
      ++m;
      if (min == 0) min = p;
    }
    ++p;
  }
  return m - min;
}

int trazi3a(int a[], int n, int x) {
  /* sekvencijalno trazenje */
  int i;
  for (i = 0; i < n; ++i)
    if (f3(a[i]) == x) return i;
  return -1;
}

int trazi3b(int a[], int n, int x) {
  /* binarno trazenje */
  /* Uvjet na niz: elementi niza moraju biti sortirani uzlazno, prema razlici broja razlicitih prostih djelitelja i najmanjeg prostog djelitelja x */
  int lijevi = 0, desni = n - 1, srednji, fa;
  while (lijevi <= desni) {
    srednji = (lijevi + desni) / 2;
    fa = f3(a[srednji]);
    if (x == fa) return srednji;
    if (x < fa) desni = srednji - 1; else lijevi = srednji + 1;
  }
  return -1;
}

/***********/
/* Grupa 4 */
/***********/

int f4(int x) {
  /* funkcija vraca najvecu kratnost medju prostim djeliteljima broja x */
  int p = 2, max = 0;
  while (x > 1) {
    if (x % p == 0) {
      int kratnost = 0;
      while (x % p == 0) {
        x /= p;
        ++kratnost;
      }
      if (kratnost > max) max = kratnost;
    }
    ++p;
  }
  return max;
}

int trazi4a(int a[], int n, int x) {
  /* sekvencijalno trazenje */
  int i;
  for (i = 0; i < n; ++i)
    if (f4(a[i]) == x) return i;
  return -1;
}

int trazi4b(int a[], int n, int x) {
  /* binarno trazenje */
  /* Uvjet na niz: elementi niza moraju biti sortirani uzlazno, prema najvecoj kratnosti medju prostim djeliteljima */
  int lijevi = 0, desni = n - 1, srednji, fa;
  while (lijevi <= desni) {
    srednji = (lijevi + desni) / 2;
    fa = f4(a[srednji]);
    if (x == fa) return srednji;
    if (x < fa) desni = srednji - 1; else lijevi = srednji + 1;
  }
  return -1;
}

/*************************************************************/
/* Glavni program (samo za testiranje; ne treba u kolokviju) */
/*************************************************************/

int main(void) {
  int niz[] = {5, 12, 16, 17, 26, 30}, n = sizeof(niz) / sizeof(int);
  report("Prva", niz, n, 7, 15, f1, trazi1a, trazi1b);
  report("Druga", niz, n, 2, 7, f2, trazi2a, trazi2b);
  report("Treca", niz, n, -16, 7, f3, trazi3a, trazi3b);
  report("Cetvrta", niz, n, 4, 7, f4, trazi4a, trazi4b);
  return 0;
}

